import {select, arc, json , geoPath , geoMercator, geoOrthographic, geoStereographic, 
        geoNaturalEarth1, zoom, tsv, event} from 'd3';
import {feature } from 'topojson';

const svg = d3.select("svg");


//const projection = geoMercator();
//const projection = geoOrthographic();
//const projection = geoStereographic();
const projection = geoNaturalEarth1();

const g= svg.append('g');

const pathGenerator = geoPath().projection(projection);
g.append('path')
			.attr('class', 'globe')
			.attr('d', pathGenerator({type: 'Sphere'}));



function zoomed() {
   svg.attr('transform', d3.event.transform);
}
svg.call(zoom().on('zoom', () => {
  g.attr('transform', event.transform)
}
	)
);

/* Use Promise.all to make sure that waits for multiple promises, before
   the callback function passed in .then() is executed 
 */
Promise.all([
	tsv('https://unpkg.com/world-atlas@1/world/110m.tsv'),
 	json('https://cdn.jsdelivr.net/npm/world-atlas@2/countries-110m.json')
  ])
	.then(([tsvData, topoJSONData]) => {
  
  /* Using reduce */
  /* reduce 
  	  1) parameters:
      	- accumulator function which is returned by reduce 
      	- the data element / row
      2) initial value for the accumulator
      
      In the first invocation the initial value is an empty object
      Every invocation will return the accumulator object.
      The accumulator object is then passed in as a parameter on
      each invocation, thus "accumulating" information for each data element.
      
  */
  
  const countryName = tsvData.reduce((accumulator, d) => {
  	accumulator[d.iso_n3]=d.name;
    return accumulator;
  }, {});
  
  /* Using forEach */
  /*
  const countryName = {};
  tsvData.forEach(d => {countryName[d.iso_n3]=d.name})
	*/
  
  const countries = feature(topoJSONData, topoJSONData.objects.countries);
  const paths=g.selectAll('path').data(countries.features);
  paths.enter().append('path')
    .attr('class', 'country')
    .attr('d', d => pathGenerator(d))
  	.append('title')
  		.text( d => countryName[d.id])
})



/*
World Atlas data (countries-110m.json) is in TopoJSON format.
D3 geo libraries can work with GeoJSON.
Therefore the TopoJSON-data needs to be converted to GeoJSON
in order to generate svg-paths using D3.
This is done with the geoPath-library.

In this script two generator functions are created:
1. projection: generate map projection
2. pathGenerator: convert TopoJSON to GeoJSON, then project it using the projection

*/






